@@ -31,18 +31,24 @@ lower (:math:`l`) leaflet.
 
 The correlation between the two leaflets, :math:`r_{u/l}`, is then calculated as
 the pearson correlation coefficient between :math:`\rho(x, y)_{u}` and
-:math:`\rho(x, y)_{l}`. For more information on interleaflet registration in bilayers
+:math:`\rho(x, y)_{l}`, where values of:
+
+* :math:`1` correspond to perfectly registered
+* :math:`-1` correspond to perfectly anti-registered
+
+For more information on interleaflet registration in bilayers
 see `Thallmair et al. (2018) <https://pubs.acs.org/doi/abs/10.1021/acs.jpclett.8b01877>`__.
 
 Input
 -----
 
 Required:
-  - *leaflets* : a :class:`lipyphilic.lib.assign_leaflets.AssignLeaflets` object
-
-Optional:
+  - *universe* : an MDAnalysis Universe object.
   - *upper_sel* : atom selection for lipids in the upper leaflet to use in the registration calculation
   - *lower_sel* : atom selection for lipids in the lower leaflet to use in the registration calculation
+  - *leaflets* : leaflet membership (-1: lower leaflet, 0: midplane, 1: upper leaflet) of each lipid in the membrane.
+
+Optional:
   - *filter_by* : boolean mask for determining which lipids to include in the registration calculation
   - *n_bins* : the number of bins to use in *x* and *y* for the 2D histogram
   - *gaussian_sd* : the standard deviation of the circular Gaussian to convole with the grid densities
@@ -61,7 +67,7 @@ Example usage of :class:`Registration`
 An MDAnalysis Universe must first be created before using :class:`Registration`::
 
   import MDAnalysis as mda
-  from lipyphilic.lib.neighbours import Neighbours
+  from lipyphilic.lib.registration import Registration
 
   u = mda.Universe(tpr, trajectory)
 
@@ -81,32 +87,30 @@ Registration object by passing our :class:`lipyphilic.lib.assign_leaflets.Assign
 object to :class:`Registration` along with atom selections for the lipids::
 
   registration = Registration(
-    leaflets=leaflets,
     upper_sel="resname CHOL and name ROH",
     lower_sel="resname CHOL and name ROH",
+    leaflets=leaflets.filter_leaflets("resname CHOL and name ROH")
   )
   
 To calculate the interleaflet correlation of cholesterol molecules using their ROH
-beads we then need to use the :func:`run()` method::
+beads we then need to use the :func:`run()` method. We select which frames of the trajectory
+to analyse (`None` will use every frame) and choose to display a progress bar (`verbose=True`)::
   
   registration.run(
+    start=None,
+    stop=None,
+    step=None,
     verbose=True
   )
 
 The results are then available in the :attr:`registration.registration` attribute as a
-:class:`numpy.ndarray`.
-
-Note
-----
-
-The same MDAndalysis Universe and the same frames of the trajectory as used in assigning
-leaflets will be used for calculating the interleaflet registration.
-
+:class:`numpy.ndarray`. Again, :math:`1` corresponds to the leaflets being perfectly in register
+and :math:`-1` corresponds to the leaflets being perfectly anti-registered.
 
 Selecting a subset of lipids for the registration analysis
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
-The previous example will compute the largest registration of cholesterol across the upper
+The previous example will compute the registration of cholesterol across the upper
 and lower leaflets. In, for example, simulations of phase-separation domains, it is useful
 to know the registration of liquid-ordered domains (regardless of the species in the domain)
 rather than the registrtion of specific lipid species.
@@ -114,11 +118,13 @@ rather than the registrtion of specific lipid species.
 If we have a 2D array, 'lipid_order_data', that contains information on whether each lipid is in
 the liquid-disordered phase or the liquid-ordered phase at each frame, we can used this to
 calculate the registration of ordered domains. The array must take the shape
-'(n_residues, n_frames)', and in the below example 'lipid_order_data[i, j]' would be equal to zero
-if lipid 'i' is liquid-disordered at frame 'j' and equal to 1 if it is liquid-ordered::
+'(n_residues, n_frames)', and in the below example 'lipid_order_data[i, j]' would be equal to `-1`
+if lipid 'i' is liquid-disordered at frame 'j' and equal to `1` if it is liquid-ordered::
 
   registration = Registration(
-    leaflets=leaflets,
+    upper_sel="name PO4 ROH",
+    lower_sel="name PO4 ROH",
+    leaflets=leaflets.leaflets,
     filter_by=lipid_order_data == 1
   )
 
@@ -127,9 +133,9 @@ If we have a ternary mixture of DPPC/DOPC/Cholesterol, we can also specifcy that
 consider only DPPC and cholesterol in the liquid-ordered phase::
 
   registration = Registration(
-    leaflets=leaflets,
     upper_sel="(resname CHOL and name ROH) or (resname DPPC and name PO4)",
     lower_sel="(resname CHOL and name ROH) or (resname DPPC and name PO4)",
+    leaflets=leaflets.filter_leaflets("resname CHOL DPPC"),
     filter_by=lipid_order_data == 1
   )
   
@@ -140,14 +146,14 @@ Changing the resolution of the 2D grid
 By default, the lipid positions of each leaflet are binned into a two-dimensional
 histogram using :math:`n\_bins_x = \lceil{x}\rceil`, where :math:`n\_bins_x` is the
 numer of bins in :math:`x` and :math:`\lceil{x}\rceil` is the size of system in :math:`x`
-rounded to the nearest integer. This gives a grid resolution of  *1* Å.
+rounded up to the nearest integer. This gives a grid resolution of  *1* Å.
 
 It is also possible to specify the number of bins to use for binning the data::
 
   registration = Registration(
-    leaflets=leaflets,
     upper_sel="resname CHOL and name ROH",
     lower_sel="resname CHOL and name ROH",
+    leaflets=leaflets.filter_leaflets("resname CHOL"),
     n_bins=100
   )
 
@@ -167,9 +173,9 @@ for determining interleaflet cholesterol correlations. This deault value can be
 changed using the :attr:`gaussian_sd` parameter::
 
   registration = Registration(
-    leaflets=leaflets,
     upper_sel="resname CHOL and name ROH",
     lower_sel="resname CHOL and name ROH",
+    leaflets=leaflets.filter_leaflets("resname CHOL"),
     gaussian_sd=12
   )
 
@@ -178,22 +184,21 @@ Figure 2d of `Thallmair et al. (2018)
 <https://pubs.acs.org/doi/abs/10.1021/acs.jpclett.8b01877>`__ shows how correlation
 tends to increase with increasing :attr:`gaussian_sd`. This is because the density of
 atomic positions is more diffuse and thus more likely to overlap between the two
-leaflets.
+leaflets. Increasing :attr:`gaussian_sd` also incurs a performance cost.
 
 The class and its methods
 -------------------------
 
 .. autoclass:: Registration
     :members:
-    :exclude-members: run
 
 """
-from tqdm.auto import tqdm
+
 import numpy as np
-import scipy
+import scipy.stats
+import scipy.ndimage
 
 from lipyphilic.lib import base
-from lipyphilic.lib.assign_leaflets import AssignLeaflets
 
 
 class Registration(base.AnalysisBase):
@@ -201,9 +206,10 @@ class Registration(base.AnalysisBase):
     """
 
     def __init__(self,
+                 universe,
+                 upper_sel,
+                 lower_sel,
                  leaflets,
-                 upper_sel=None,
-                 lower_sel=None,
                  filter_by=None,
                  n_bins=None,
                  gaussian_sd=15
@@ -212,17 +218,22 @@ class Registration(base.AnalysisBase):
         
         Parameters
         ----------
-        leaflets : AssignLeaflets
-            An AssignLeaflets object that contains the leaflet membership for each lipid
-            at each frame. See lipyphilic.lib.assign_leaflets.AssignLeaflets for details.
-        upper_sel : str, optional
+        universe : Universe
+            MDAnalysis Universe object
+        upper_sel : str
             Selection string for lipids in the upper leaflet of the bilayer to be used
-            for determining registration. The default is `None`, in which case all atoms
-            of all lipids in the upper leaflet will be used.
-        lower_sel : str, optional
+            for determining registration.
+        lower_sel : str
             Selection string for lipids in the lower leaflet of the bilayer to be used
-            for determining registration. The default is `None`, in which case all atoms
-            of all lipids in the lower leaflet will be used.
+            for determining registration.
+        leaflets : numpy.ndarray
+            An array of leaflet membership in which: -1 corresponds to the lower leaflet;
+            1 corresponds to the upper leaflet; and 0 corresponds to the midplane.
+            If the array is 1D and of shape (n_lipids), each lipid is taken to
+            remain in the same leaflet over the trajectory. If the array is 2D and
+            of shape (n_lipids, n_frames), the leaflet to which each lipid is
+            assisgned at each frame will be taken into account when calculating
+            the area per lipid.
         filter_by : numpy.ndarray, optional
             A boolean array indicating whether or not to include each lipid in the registration
             analysis. If the array is 1D and of shape (n_lipids), the same lipids will be used
@@ -240,20 +251,26 @@ class Registration(base.AnalysisBase):
             size of the lipids. The default is 15.
         """
         
-        if not isinstance(leaflets, AssignLeaflets):
-            raise ValueError("leaflets must be of type AssignLeaflets")
+        super(Registration, self).__init__(universe.trajectory)
         
-        super(Registration, self).__init__(leaflets.u.trajectory)
+        self.u = universe
+        self.upper_sel = upper_sel
+        self.lower_sel = lower_sel
+        self.membrane = self.u.select_atoms(f"({self.upper_sel}) or ({self.lower_sel})")
         
-        self.u = leaflets.u
-        self.membrane = leaflets.membrane
-        self.leaflets = leaflets.leaflets
-        self.start = leaflets.start
-        self.stop = leaflets.stop
-        self.step = leaflets.step
+        if np.array(leaflets).ndim not in [1, 2]:
+            raise ValueError("'leaflets' must either be a 1D array containing non-changing "
+                             "leaflet ids of each lipid, or a 2D array of shape (n_residues, n_frames)"
+                             " containing the leaflet id of each lipid at each frame."
+                             )
+
+        if len(leaflets) != self.membrane.n_residues:
+            raise ValueError("The shape of 'leaflets' must be (n_residues,), but 'lipid_sel' "
+                             f"generates an AtomGroup with {self.membrane.n_residues} residues"
+                             f" and 'leaflets' has shape {leaflets.shape}."
+                             )
         
-        self.upper_sel = upper_sel if upper_sel is not None else "all"
-        self.lower_sel = lower_sel if lower_sel is not None else "all"
+        self.leaflets = leaflets
         
         if filter_by is not None and np.array(filter_by).ndim not in [1, 2]:
             raise ValueError("'filter_by' must either be a 1D array containing non-changing boolean"
@@ -284,16 +301,8 @@ class Registration(base.AnalysisBase):
                 
         self.n_bins = n_bins
         self.gaussian_sd = gaussian_sd
-    
-    def _setup_frames(self):
-        """
-        Pass a Reader object and define the desired iteration pattern
-        through the trajectory
-        """
         
-        self.n_frames = len(range(self.start, self.stop, self.step))
-        self.frames = np.zeros(self.n_frames, dtype=int)
-        self.times = np.zeros(self.n_frames)
+        self.registration = None
     
     def _prepare(self):
         
@@ -355,24 +364,3 @@ class Registration(base.AnalysisBase):
             lower_hist.flatten()
         )
         self.registration[self._frame_index] = correlation[0]
-        
-    def run(self):
-        """Perform the calculation
-        """
-        
-        self._setup_frames()
-        self._prepare()
-        
-        for i, ts in tqdm(enumerate(
-                self._trajectory[self.start:self.stop:self.step]),
-                total=self.n_frames):
-            
-            self._frame_index = i
-            self._ts = ts
-            self.frames[i] = ts.frame
-            self.times[i] = ts.time
-            
-            self._single_frame()
-        
-        self._conclude()
-        return self
